@@ -6,21 +6,20 @@
 #include "core/feeddownloader.h"
 #include "database/databasefactory.h"
 #include "definitions/definitions.h"
-#include "gui/systemtrayicon.h"
+#include "gui/tray/trayicon.h"
 #include "miscellaneous/feedreader.h"
 #include "miscellaneous/iofactory.h"
 #include "miscellaneous/localization.h"
-#include "miscellaneous/nodejs.h"
 #include "miscellaneous/notification.h"
 #include "miscellaneous/singleapplication.h"
 #include "miscellaneous/skinfactory.h"
 #include "miscellaneous/systemfactory.h"
-#include "network-web/downloadmanager.h"
 
 #include <functional>
 
 #include <QCommandLineParser>
 #include <QList>
+#include <QSystemTrayIcon>
 
 #if defined(qApp)
 #undef qApp
@@ -32,15 +31,7 @@
 class FormMain;
 class FormLog;
 class IconFactory;
-class QAction;
 class Mutex;
-
-#if QT_VERSION_MAJOR == 6
-class QWebEngineDownloadRequest;
-#else
-class QWebEngineDownloadItem;
-#endif
-
 class WebFactory;
 class NotificationFactory;
 class ToastNotificationsManager;
@@ -97,6 +88,9 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     explicit Application(const QString& id, int& argc, char** argv, const QStringList& raw_cli_args);
     virtual ~Application();
 
+    virtual bool event(QEvent* event);
+
+    void updateCliDebugStatus();
     void reactOnForeignNotifications();
     void hideOrShowMainForm();
     void loadDynamicShortcuts();
@@ -104,7 +98,6 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     void offerChanges() const;
 
     bool isAlreadyRunning();
-
     QStringList builtinSounds() const;
 
     FeedReader* feedReader();
@@ -129,15 +122,17 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     Localization* localization();
     DatabaseFactory* database();
     IconFactory* icons();
-    DownloadManager* downloadManager();
     Settings* settings() const;
     Mutex* feedUpdateLock();
     FormMain* mainForm();
     QWidget* mainFormWidget();
-    SystemTrayIcon* trayIcon();
+    TrayIcon* trayIcon();
     NotificationFactory* notifications() const;
-    NodeJs* nodejs() const;
+
+#if QT_VERSION_MAJOR > 5
     QThreadPool* workHorsePool() const;
+#endif
+
     ToastNotificationsManager* toastNotifications() const;
 
     QIcon desktopAwareIcon() const;
@@ -156,11 +151,7 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     // NOTE: Use this to get correct path under which store user data.
     QString userDataFolder();
 
-    QString cacheFolder();
-
-    int customAdblockPort() const;
-
-    QString replaceUserDataFolderPlaceholder(QString text) const;
+    QString replaceUserDataFolderPlaceholder(QString text, bool double_escape = false) const;
     QStringList replaceUserDataFolderPlaceholder(QStringList texts) const;
 
     void setMainForm(FormMain* main_form);
@@ -188,14 +179,8 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
                         QWidget* parent = nullptr);
 
     WebViewer* createWebView();
-
-    bool usingLite() const;
     bool isWayland() const;
 
-#if defined(NO_LITE)
-    bool forcedLite() const;
-#endif
-
     // Returns pointer to "GOD" application singleton.
     static Application* instance();
 
@@ -203,10 +188,8 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     static void performLogging(QtMsgType type, const QMessageLogContext& context, const QString& msg);
 
   public slots:
-    // Restarts the application.
-    void restart();
-
-    void reloadCurrentSkin(bool replace_existing_qss = true);
+    void setupFont();
+    void reloadCurrentSkin(bool replace_existing_qss);
 
     // Processes incoming message from another RSS Guard instance.
     void parseCmdArgumentsFromOtherInstance(const QString& message);
@@ -223,27 +206,10 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
   private slots:
     void loadMessageToFeedAndArticleList(Feed* feed, const Message& message);
     void fillCmdArgumentsParser(QCommandLineParser& parser);
-
-    void onNodeJsPackageUpdateError(const QObject* sndr,
-                                    const QList<NodeJs::PackageMetadata>& pkgs,
-                                    const QString& error);
-    void onNodeJsPackageInstalled(const QObject* sndr,
-                                  const QList<NodeJs::PackageMetadata>& pkgs,
-                                  bool already_up_to_date);
     void onCommitData(QSessionManager& manager);
     void onSaveState(QSessionManager& manager);
     void onAboutToQuit();
     void showMessagesNumber(int unread_messages, bool any_feed_has_new_unread_messages);
-    void onAdBlockFailure();
-
-#if defined(NO_LITE)
-#if QT_VERSION_MAJOR == 6
-    void downloadRequested(QWebEngineDownloadRequest* download_item);
-#else
-    void downloadRequested(QWebEngineDownloadItem* download_item);
-#endif
-#endif
-
     void onFeedUpdatesStarted();
     void onFeedUpdatesProgress(const Feed* feed, int current, int total);
     void onFeedUpdatesFinished(const FeedDownloadResults& results);
@@ -256,11 +222,11 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     QImage generateOverlayIcon(int number) const;
 #endif
 
-    void setupFontAntialiasing();
     void setupCustomDataFolder(const QString& data_folder);
     void setupWorkHorsePool();
     void determineFirstRuns();
     void eliminateFirstRuns();
+    void initializeFileBasedLogging();
     void displayLogMessageInDialog(const QString& message);
 
   private:
@@ -287,7 +253,7 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     QList<QAction*> m_userActions;
     FormMain* m_mainForm;
     FormLog* m_logForm;
-    SystemTrayIcon* m_trayIcon;
+    TrayIcon* m_trayIcon;
     Settings* m_settings;
     WebFactory* m_webFactory;
     SystemFactory* m_system;
@@ -295,22 +261,18 @@ class RSSGUARD_DLLSPEC Application : public SingleApplication {
     Localization* m_localization;
     IconFactory* m_icons;
     DatabaseFactory* m_database;
-    DownloadManager* m_downloadManager;
     NotificationFactory* m_notifications;
     ToastNotificationsManager* m_toastNotifications;
-    NodeJs* m_nodejs;
+
+#if QT_VERSION_MAJOR > 5
     QThreadPool* m_workHorsePool;
-    bool m_shouldRestart;
+#endif
+
     bool m_firstRunEver;
     bool m_firstRunCurrentVersion;
     QString m_customDataFolder;
-    int m_customAdblockPort;
     bool m_allowMultipleInstances;
 
-#if defined(NO_LITE)
-    bool m_forcedLite;
-#endif
-
 #if defined(Q_OS_WIN)
     ITaskbarList4* m_windowsTaskBar;
 #endif
