@@ -11,10 +11,9 @@ class RSSGUARD_DLLSPEC ImportantNode : public RootItem {
   public:
     explicit ImportantNode(RootItem* parent_item = nullptr);
 
-    virtual QList<Message> undeletedMessages() const;
-    virtual bool cleanMessages(bool clean_read_only);
-    virtual void updateCounts(bool including_total_count);
-    virtual bool markAsReadUnread(ReadStatus status);
+    virtual void cleanMessages(bool clean_read_only);
+    virtual void updateCounts();
+    virtual void markAsReadUnread(ReadStatus status);
     virtual int countOfUnreadMessages() const;
     virtual int countOfAllMessages() const;
 
