@@ -15,17 +15,15 @@ class RSSGUARD_DLLSPEC LabelsNode : public RootItem {
     QList<Label*> labels() const;
     void loadLabels(const QList<Label*>& labels);
 
-    virtual QList<Message> undeletedMessages() const;
-    virtual QList<QAction*> contextMenuFeedsList();
-    virtual void updateCounts(bool including_total_count);
+    virtual void markAsReadUnread(RootItem::ReadStatus status);
+    virtual QString additionalTooltip() const;
+    virtual void updateCounts();
 
-    Label* labelById(const QString& custom_id);
+    Label* labelByCustomId(const QString& custom_id);
+    QHash<QString, Label*> getHashedLabels() const;
 
   public slots:
     void createLabel();
-
-  private:
-    QAction* m_actLabelNew;
 };
 
 #endif // LABELSNODE_H
