@@ -10,15 +10,18 @@
 class RSSGUARD_DLLSPEC Search : public RootItem {
     Q_OBJECT
 
-    // Added for message filtering with labels.
-    Q_PROPERTY(QColor color READ color)
-
   public:
-    explicit Search(const QString& name, const QString& filter, const QColor& color, RootItem* parent_item = nullptr);
-    explicit Search(RootItem* parent_item = nullptr);
+    enum class Type {
+      Regex = 0,
+      SqlWhereClause = 1
+    };
 
-    QColor color() const;
-    void setColor(const QColor& color);
+    explicit Search(const QString& name,
+                    Type type,
+                    const QString& filter,
+                    const QIcon& icon,
+                    RootItem* parent_item = nullptr);
+    explicit Search(RootItem* parent_item = nullptr);
 
     QString filter() const;
     void setFilter(const QString& new_filter);
@@ -26,20 +29,22 @@ class RSSGUARD_DLLSPEC Search : public RootItem {
     void setCountOfAllMessages(int totalCount);
     void setCountOfUnreadMessages(int unreadCount);
 
-    virtual bool cleanMessages(bool clear_only_read);
+    virtual void cleanMessages(bool clear_only_read);
     virtual QString additionalTooltip() const;
-    virtual bool markAsReadUnread(ReadStatus status);
+    virtual void markAsReadUnread(ReadStatus status);
     virtual int countOfAllMessages() const;
     virtual int countOfUnreadMessages() const;
     virtual bool canBeEdited() const;
     virtual bool canBeDeleted() const;
-    virtual bool deleteItem();
-    virtual void updateCounts(bool including_total_count);
-    virtual QList<Message> undeletedMessages() const;
+    virtual void deleteItem();
+    virtual void updateCounts();
+
+    Type type() const;
+    void setType(Type kind);
 
   private:
+    Type m_type;
     QString m_filter;
-    QColor m_color;
     int m_totalCount = -1;
     int m_unreadCount = -1;
 };
