@@ -3,6 +3,7 @@
 #ifndef SERVICEROOT_H
 #define SERVICEROOT_H
 
+#include "core/feedsmodel.h"
 #include "core/message.h"
 #include "definitions/typedefs.h"
 #include "services/abstract/rootitem.h"
@@ -13,7 +14,6 @@
 
 class QAction;
 class QMutex;
-class FeedsModel;
 class RecycleBin;
 class ImportantNode;
 class UnreadNode;
@@ -24,6 +24,7 @@ class MessagesModel;
 class CustomMessagePreviewer;
 class CacheForServiceRoot;
 class FormAccountDetails;
+class QMenu;
 
 // THIS IS the root node of the service.
 // NOTE: The root usually contains some core functionality of the
@@ -61,12 +62,13 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
 
     virtual FormAccountDetails* accountSetupDialog() const;
     virtual void onDatabaseCleanup();
-    virtual void updateCounts(bool including_total_count);
+    virtual void updateCounts();
     virtual bool canBeDeleted() const;
-    virtual bool deleteItem();
+    virtual void deleteItem();
+    virtual void updateItemTitle(RootItem* item, const QString& new_title);
     virtual void editItems(const QList<RootItem*>& items);
-    virtual bool markAsReadUnread(ReadStatus status);
-    virtual QList<Message> undeletedMessages() const;
+    virtual void markAsReadUnread(ReadStatus status);
+    virtual void cleanMessages(bool clean_read_only);
     virtual bool supportsFeedAdding() const;
     virtual bool supportsCategoryAdding() const;
     virtual LabelOperation supportedLabelOperations() const;
@@ -90,7 +92,7 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     virtual QList<QAction*> addItemMenu();
 
     // NOTE: Caller does NOT take ownership of created menu/actions!
-    virtual QList<QAction*> contextMenuFeedsList();
+    virtual QList<QAction*> contextMenuFeedsList(const QList<RootItem*>& selected_items);
 
     // NOTE: Caller does NOT take ownership of created menu/actions!
     virtual QList<QAction*> contextMenuMessagesList(const QList<Message>& messages);
@@ -129,73 +131,41 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     // This method should load messages for given "item" into model.
     virtual bool loadMessagesForItem(RootItem* item, MessagesModel* model);
 
-    // Called BEFORE this read status update (triggered by user in message list) is stored in DB,
-    // when false is returned, change is aborted.
+    // These are called BEFORE and AFTER this article status update is stored in DB.
     // This is the place to make some other changes like updating
     // some ONLINE service or something.
-    //
-    // "read" is status which is ABOUT TO BE SET.
-    virtual bool onBeforeSetMessagesRead(RootItem* selected_item, const QList<Message>& messages, ReadStatus read);
 
-    // Called AFTER this read status update (triggered by user in message list) is stored in DB,
-    // when false is returned, change is aborted.
-    // Here service root should inform (via signals)
-    // which items are actually changed.
-    //
-    // "read" is status which is ABOUT TO BE SET.
-    virtual bool onAfterSetMessagesRead(RootItem* selected_item, const QList<Message>& messages, ReadStatus read);
+    virtual void onBeforeSetMessagesRead(RootItem* selected_item,
+                                         const QStringList& message_custom_ids,
+                                         ReadStatus read);
+    virtual void onBeforeSetMessagesRead(RootItem* selected_item, const QList<Message>& messages, ReadStatus read);
+    virtual void onAfterSetMessagesRead(RootItem* selected_item, const QList<Message>& messages, ReadStatus read);
 
-    // Called BEFORE this importance switch update is stored in DB,
-    // when false is returned, change is aborted.
-    // This is the place to make some other changes like updating
-    // some ONLINE service or something.
-    //
-    // "changes" - list of pairs - <message (integer id), new status>
-    virtual bool onBeforeSwitchMessageImportance(RootItem* selected_item, const QList<ImportanceChange>& changes);
+    virtual void onBeforeSwitchMessageImportance(RootItem* selected_item, const QList<ImportanceChange>& changes);
+    virtual void onAfterSwitchMessageImportance(RootItem* selected_item, const QList<ImportanceChange>& changes);
 
-    // Called AFTER this importance switch update is stored in DB,
-    // when false is returned, change is aborted.
-    // Here service root should inform (via signals)
-    // which items are actually changed.
-    //
-    // "changes" - list of pairs - <message (integer id), new status>
-    virtual bool onAfterSwitchMessageImportance(RootItem* selected_item, const QList<ImportanceChange>& changes);
+    virtual void onBeforeMessagesDelete(RootItem* selected_item, const QList<Message>& messages);
+    virtual void onAfterMessagesDelete(RootItem* selected_item, const QList<Message>& messages);
 
-    // Called BEFORE the list of messages is about to be deleted
-    // by the user from message list.
-    virtual bool onBeforeMessagesDelete(RootItem* selected_item, const QList<Message>& messages);
-
-    // Called AFTER the list of messages was deleted
-    // by the user from message list.
-    virtual bool onAfterMessagesDelete(RootItem* selected_item, const QList<Message>& messages);
-
-    // Called BEFORE some labels are assigned/deassigned from/to messages.
-    virtual bool onBeforeLabelMessageAssignmentChanged(const QList<Label*>& labels,
+    virtual void onBeforeLabelMessageAssignmentChanged(const QList<Label*>& labels,
                                                        const QList<Message>& messages,
                                                        bool assign);
-
-    // Called AFTER some labels are assigned/deassigned from/to messages.
-    virtual bool onAfterLabelMessageAssignmentChanged(const QList<Label*>& labels,
+    virtual void onAfterLabelMessageAssignmentChanged(const QList<Label*>& labels,
                                                       const QList<Message>& messages,
                                                       bool assign);
 
-    // Called BEFORE the list of messages is about to be restored from recycle bin
-    // by the user from message list.
-    // Selected item is naturally recycle bin.
-    virtual bool onBeforeMessagesRestoredFromBin(RootItem* selected_item, const QList<Message>& messages);
+    virtual void onBeforeMessagesRestoredFromBin(RootItem* selected_item, const QList<Message>& messages);
+    virtual void onAfterMessagesRestoredFromBin(RootItem* selected_item, const QList<Message>& messages);
 
-    // Called AFTER the list of messages was restored from recycle bin
-    // by the user from message list.
-    // Selected item is naturally recycle bin.
-    virtual bool onAfterMessagesRestoredFromBin(RootItem* selected_item, const QList<Message>& messages);
-
-    // Called AFTER articles of these feeds are purged from the database.
     virtual void onAfterFeedsPurged(const QList<Feed*>& feeds);
 
     // Returns the UNIQUE code of the given service.
     // NOTE: Keep in sync with ServiceEntryRoot::code().
     virtual QString code() const = 0;
 
+    // Provides abstraction for service-wide or feed-wide proxy.
+    virtual QNetworkProxy networkProxyForItem(RootItem* item) const;
+
   public:
     CacheForServiceRoot* toCache() const;
 
@@ -211,27 +181,21 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     void completelyRemoveAllData();
 
     // Returns counts of updated messages <unread, all>.
-    UpdatedArticles updateMessages(QList<Message>& messages, Feed* feed, bool force_update, QMutex* db_mutex);
-
-    QIcon feedIconForMessage(const QString& feed_custom_id) const;
+    UpdatedArticles updateMessages(QList<Message>& messages, Feed* feed, bool force_update, bool recalculate_counts);
 
     // Removes all/read only messages from given underlying feeds.
-    bool cleanFeeds(const QList<Feed*>& items, bool clean_read_only);
-
-    // Marks all messages from feeds read/unread.
-    bool markFeedsReadUnread(const QList<Feed*>& items, ReadStatus read);
+    void cleanFeeds(const QList<Feed*>& items, bool clean_read_only);
 
     // Obvious methods to wrap signals.
     void itemChanged(const QList<RootItem*>& items);
-    void requestReloadMessageList(bool mark_selected_messages_read);
+    void informOthersAboutDataChange(RootItem* item, FeedsModel::ExternalDataChange change);
     void requestItemExpand(const QList<RootItem*>& items, bool expand);
     void requestItemExpandStateSave(RootItem* subtree_root);
-    void requestItemReassignment(RootItem* item, RootItem* new_parent);
+    void requestItemReassignment(RootItem* item, RootItem* new_parent, bool blocking = false);
     void requestItemsReassignment(const QList<RootItem*>& items, RootItem* new_parent);
-    void requestItemRemoval(RootItem* item);
+    void requestItemRemoval(RootItem* item, bool reload_counts);
 
     // Some message/feed attribute selectors.
-    QStringList textualFeedUrls(const QList<Feed*>& feeds) const;
     QStringList textualFeedIds(const QList<Feed*>& feeds) const;
     QStringList customIDsOfMessages(const QList<ImportanceChange>& changes);
     QStringList customIDsOfMessages(const QList<Message>& messages);
@@ -242,7 +206,7 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     // only IDs when the change makes sense.
     // NOTE: Importance is not dealt here because it was not needed
     // yet.
-    QStringList customIDSOfMessagesForItem(RootItem* item,
+    QStringList customIDsOfMessagesForItem(RootItem* item,
                                            RootItem::ReadStatus target_read = RootItem::ReadStatus::Unknown);
 
     void performInitialAssembly(const Assignment& categories,
@@ -273,39 +237,26 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
 
     // Removes all messages/categories/feeds which are
     // associated with this account.
-    void removeOldAccountFromDatabase(bool delete_messages_too, bool delete_labels_too);
     void cleanAllItemsFromModel(bool clean_labels_too);
     void appendCommonNodes();
 
-    // Removes messages which do not belong to any
-    // existing feed.
-    //
-    // NOTE: This situation may happen if user deletes some feed
-    // from another machine and then performs sync-in on this machine.
-    void removeLeftOverMessages();
-
-    // Removes all msg. filter assignments which are (within this account)
-    // assigned to feed (via custom ID) which does not exist anymore.
-    //
-    // NOTE: This situation may happen if user deletes some feed
-    // from another machine and then performs sync-in on this machine.
-    void removeLeftOverMessageFilterAssignments();
-
-    // Takes lists of feeds/categories and assembles them into the tree structure.
-    void assembleCategories(const Assignment& categories);
-    void assembleFeeds(const Assignment& feeds);
-
   signals:
     void proxyChanged(QNetworkProxy proxy);
     void dataChanged(QList<RootItem*> items);
-    void reloadMessageListRequested(bool mark_selected_messages_read);
+    void dataChangeNotificationTriggered(RootItem* item, FeedsModel::ExternalDataChange change);
     void itemExpandRequested(QList<RootItem*> items, bool expand);
     void itemExpandStateSaveRequested(RootItem* subtree_root);
 
+    void itemBlockingReassignmentRequested(RootItem* item, RootItem* new_parent);
     void itemReassignmentRequested(RootItem* item, RootItem* new_parent);
-    void itemRemovalRequested(RootItem* item);
+    void itemRemovalRequested(RootItem* item, bool reload_counts);
 
   private:
+    void refreshAfterArticlesChange(const QList<Feed*>& feeds,
+                                    const QList<Message>& messages,
+                                    bool refresh_bin,
+                                    bool refresh_only_bin);
+
     void resortAccountTree(RootItem* tree,
                            const QMap<QString, QVariantMap>& custom_category_data,
                            const QMap<QString, QVariantMap>& custom_feed_data) const;
@@ -316,9 +267,13 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     // Key is category's custom ID.
     virtual QMap<QString, QVariantMap> storeCustomCategoriesData();
 
+    // Key is label's custom ID.
+    virtual QMap<QString, QVariantMap> storeCustomLabelsData();
+
     virtual void restoreCustomFeedsData(const QMap<QString, QVariantMap>& data, const QHash<QString, Feed*>& feeds);
     virtual void restoreCustomCategoriesData(const QMap<QString, QVariantMap>& data,
                                              const QHash<QString, Category*>& cats);
+    virtual void restoreCustomLabelsData(const QMap<QString, QVariantMap>& data, LabelsNode* labels);
 
   protected:
     RecycleBin* m_recycleBin;
@@ -333,6 +288,9 @@ class RSSGUARD_DLLSPEC ServiceRoot : public RootItem {
     bool m_nodeShowImportant;
     bool m_nodeShowLabels;
     bool m_nodeShowProbes;
+    QList<QAction*> m_contextMenuProbes;
+    QList<QAction*> m_contextMenuBin;
+    QList<QAction*> m_contextMenuLabels;
 };
 
 #if QT_VERSION_MAJOR == 6
